@@ -100,7 +100,7 @@ static pa_hook_result_t sink_put_hook_callback(pa_core *c, pa_sink *sink, void*
 
     /* No default sink, nothing to move away, just set the new default */
     if (!c->default_sink) {
-        pa_core_set_configured_default_sink(c, sink->name);
+        pa_core_set_policy_default_sink(c, sink->name);
         return PA_HOOK_OK;
     }
 
@@ -116,7 +116,7 @@ static pa_hook_result_t sink_put_hook_callback(pa_core *c, pa_sink *sink, void*
         }
 
     /* Actually do the switch to the new sink */
-    pa_core_set_configured_default_sink(c, sink->name);
+    pa_core_set_policy_default_sink(c, sink->name);
 
     return PA_HOOK_OK;
 }
@@ -160,7 +160,7 @@ static pa_hook_result_t source_put_hook_callback(pa_core *c, pa_source *source,
 
     /* No default source, nothing to move away, just set the new default */
     if (!c->default_source) {
-        pa_core_set_configured_default_source(c, source->name);
+        pa_core_set_policy_default_source(c, source->name);
         return PA_HOOK_OK;
     }
 
@@ -176,7 +176,7 @@ static pa_hook_result_t source_put_hook_callback(pa_core *c, pa_source *source,
         }
 
     /* Actually do the switch to the new source */
-    pa_core_set_configured_default_source(c, source->name);
+    pa_core_set_policy_default_source(c, source->name);
 
     return PA_HOOK_OK;
 }
