@@ -30,13 +30,13 @@
 typedef struct pa_rtp_context pa_rtp_context;
 
 int pa_rtp_context_init_send(pa_rtp_context *c, int fd, uint8_t payload, size_t mtu, size_t frame_size);
-pa_rtp_context* pa_rtp_context_new_send(int fd, uint8_t payload, size_t mtu, const pa_sample_spec *ss);
+pa_rtp_context* pa_rtp_context_new_send(int fd, uint8_t payload, size_t mtu, const pa_sample_spec *ss, bool enable_opus);
 
 /* If the memblockq doesn't have a silence memchunk set, then the caller must
  * guarantee that the current read index doesn't point to a hole. */
 int pa_rtp_send(pa_rtp_context *c, pa_memblockq *q);
 
-pa_rtp_context* pa_rtp_context_new_recv(int fd, uint8_t payload, const pa_sample_spec *ss);
+pa_rtp_context* pa_rtp_context_new_recv(int fd, uint8_t payload, const pa_sample_spec *ss, bool enable_opus);
 int pa_rtp_recv(pa_rtp_context *c, pa_memchunk *chunk, pa_mempool *pool, uint32_t *rtp_tstamp, struct timeval *tstamp);
 
 void pa_rtp_context_free(pa_rtp_context *c);
@@ -44,13 +44,13 @@ void pa_rtp_context_free(pa_rtp_context *c);
 size_t pa_rtp_context_get_frame_size(pa_rtp_context *c);
 pa_rtpoll_item* pa_rtp_context_get_rtpoll_item(pa_rtp_context *c, pa_rtpoll *rtpoll);
 
-pa_sample_spec* pa_rtp_sample_spec_fixup(pa_sample_spec *ss);
+pa_sample_spec* pa_rtp_sample_spec_fixup(pa_sample_spec *ss, bool enable_opus);
 int pa_rtp_sample_spec_valid(const pa_sample_spec *ss);
 
 uint8_t pa_rtp_payload_from_sample_spec(const pa_sample_spec *ss);
 pa_sample_spec *pa_rtp_sample_spec_from_payload(uint8_t payload, pa_sample_spec *ss);
 
 const char* pa_rtp_format_to_string(pa_sample_format_t f);
-pa_sample_format_t pa_rtp_string_to_format(const char *s);
+pa_sample_format_t pa_rtp_string_to_format(const char *s, bool enable_opus);
 
 #endif
