@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 /***
@@ -26,6 +24,8 @@
 
 #if defined(HAVE_POLL_H)
 #include <poll.h>
+#elif OS_IS_WIN32 && HAVE_WINSOCK2_H && (_WIN32_WINNT >= 0x0600)
+#include <winsock2.h>
 #else
 
 /* Event types that can be polled for.  These bits may be set in `events'
